from abc import ABC, abstractmethod
from typing import List
from pydantic import BaseModel
from pyspedfiscal.exception import ValorCampoInvalido, LinhaRegistroInvalida, TamanhoRegistroInvalido


class Registro(BaseModel, ABC):
    @classmethod
    def ler_registro(cls, linha: str):
        valores = linha.split("|")[1:-1]

        # Todos os atributos da classe
        campos = list(cls.__fields__.keys())

        # Exclui os atributos com exclude=True
        # Geralmente os filhos
        if cls.__exclude_fields__:
            campos = [
                campo for campo in campos if campo not in cls.__exclude_fields__]

        # Checa o o tamnho do registro
        if len(campos) != len(valores):
            raise TamanhoRegistroInvalido(
                f'Registro com tamanho inválido. \n {linha}')

        valores_dict = dict(list(zip(campos, valores)))

        try:
            return cls(**valores_dict)
        except ValorCampoInvalido as e:
            raise LinhaRegistroInvalida(f'Linha invalida.\n{linha}\n{e}')

    def __str__(self) -> str:
        return super().__str__()


class Bloco(BaseModel, ABC):

    @abstractmethod
    def ler_registros(self, registros: List[str]):
        pass

    def __setattr__(self, name, value):
        self.__dict__[name] = value



from typing import List, Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoEnumerate
from pydantic import Field

from pyspedfiscal.bloco_c.registro_c100 import RegistroC100


class IndMov(CampoEnumerate):
    """ Indicador de movimento """
    com_movimento = '0'
    sem_movimento = '1'


class RegistroC001(Registro):
    """ ABERTURA DO BLOCO C """
    reg: Literal['C001']
    ind_mov: IndMov

    # Registros filhos nivel 2
    registros_c100: List[RegistroC100] = Field(
        default_factory=list, exclude=True)



from typing import List
from ..models import Bloco
from .registro_0000 import Registro0000
from .registro_0001 import Registro0001
from .registro_0002 import Registro0002
from .registro_0005 import Registro0005
from .registro_0015 import Registro0015
from .registro_0100 import Registro0100
from .registro_0150 import Registro0150
from .registro_0175 import Registro0175
from .registro_0190 import Registro0190
from .registro_0200 import Registro0200
from .registro_0205 import Registro0205
from .registro_0206 import Registro0206
from .registro_0210 import Registro0210
from .registro_0220 import Registro0220
from .registro_0221 import Registro0221
from .registro_0300 import Registro0300
from .registro_0305 import Registro0305
from .registro_0400 import Registro0400
from .registro_0450 import Registro0450
from .registro_0460 import Registro0460
from .registro_0500 import Registro0500
from .registro_0600 import Registro0600
from .registro_0990 import Registro0990


class Bloco0(Bloco):
    def __init__(self):
        self.registro_0000: Registro0000 = None
        self.registro_0001: Registro0001 = None
        self.registro_0990: Registro0990 = None

    @classmethod
    def ler_registros(cls, registros: List[str]) -> 'Bloco0':
        bloco = cls()

        for registro in registros:
            tipo_registro = registro[1:5].upper()

            # Nivel 0 - Abertura do Arquivo Digital
            if tipo_registro == '0000':
                bloco.registro_0000 = Registro0000.ler_registro(registro)

            # Nivel 1 - Registro Pai
            if tipo_registro == '0001':
                bloco.registro_0001 = Registro0001.ler_registro(registro)

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0002':
                bloco.registro_0001.registro_0002 = Registro0002.ler_registro(registro)

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0005':
                bloco.registro_0001.registro_0005 = Registro0005.ler_registro(registro)

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0015':
                bloco.registro_0001.registros_0015.append(Registro0015.ler_registro(registro))

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0100':
                bloco.registro_0001.registro_0100 = Registro0100.ler_registro(registro)

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0150':
                bloco.registro_0001.registros_0150.append(Registro0150.ler_registro(registro))

            # Nivel 3 - Filho de 0150
            if tipo_registro == '0175':
                bloco.registro_0001.registros_0150[-1].registros_0175.append(
                    Registro0175.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0190':
                bloco.registro_0001.registros_0190.append(
                    Registro0190.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0200':
                bloco.registro_0001.registros_0200.append(
                    Registro0200.ler_registro(registro)
                )

            # Nivel 3 - Filho de 0200
            if tipo_registro == '0205':
                bloco.registro_0001.registros_0200[-1].registros_0205.append(
                    Registro0205.ler_registro(registro)
                )

            # Nivel 3 - Filho de 0200
            if tipo_registro == '0206':
                bloco.registro_0001.registros_0200[-1].registros_0206.append(
                    Registro0206.ler_registro(registro)
                )

            # Nivel 3 - Filho de 0200
            if tipo_registro == '0210':
                bloco.registro_0001.registros_0200[-1].registros_0210.append(
                    Registro0210.ler_registro(registro)
                )

            # Nivel 3 - Filho de 0200
            if tipo_registro == '0220':
                bloco.registro_0001.registros_0200[-1].registros_0220.append(
                    Registro0220.ler_registro(registro)
                )

            # Nivel 3 - Filho de 0200
            if tipo_registro == '0221':
                bloco.registro_0001.registros_0200[-1].registros_0221.append(
                    Registro0221.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0300':
                bloco.registro_0001.registros_0300.append(
                    Registro0300.ler_registro(registro)
                )

            # Nivel 3 - Filho de 0300
            if tipo_registro == '0305':
                bloco.registro_0001.registros_0300[-1].registros_0305.append(
                    Registro0305.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0400':
                bloco.registro_0001.registros_0400.append(
                    Registro0400.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0450':
                bloco.registro_0001.registros_0450.append(
                    Registro0450.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0460':
                bloco.registro_0001.registros_0460.append(
                    Registro0460.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0500':
                bloco.registro_0001.registros_0500.append(
                    Registro0500.ler_registro(registro)
                )

            # Nivel 2 - Filho de 0001
            if tipo_registro == '0600':
                bloco.registro_0001.registros_0600.append(
                    Registro0600.ler_registro(registro)
                )

            # Nivel 1 - Encerramento do Bloco 0
            if tipo_registro == '0990':
                bloco.registro_0990 = Registro0990.ler_registro(registro)

        return bloco



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico


class Registro0206(Registro):
    """ CÓDIGO DE PRODUTO CONFORME TABELA PUBLICADA PELA ANP """

    reg: Literal['0206']
    cod_comb: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoData


class Registro0175(Registro):
    """ Alteração da Tabela de Cadastro de Participante  """

    reg: Literal['0175']
    dt_alt: CampoData
    nr_campo: CampoAlphanumerico
    cont_ant: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.campos import CampoDecimal, CampoInteiro, CampoAlphanumerico, CampoEnumerate
from pydantic import BaseModel, Field
from pyspedfiscal.models import Registro


class IndMov(CampoEnumerate):
    """ Movimentação física do ITEM/PRODUTO: """
    sim = '0'
    nao = '1'


class IndAPur(CampoEnumerate):
    """ Indicador de período de apuração do IPI """
    mensal = '0'
    decendial = '1'


class RegistroC170(Registro):
    """ ITENS DO DOCUMENTO (CÓDIGO 01, 1B, 04 e 55) """

    reg: Literal['C170']
    num_item: CampoInteiro
    cod_item: CampoAlphanumerico
    descr_compl: CampoAlphanumerico
    qtd: CampoDecimal
    unid: CampoAlphanumerico
    vl_item: CampoDecimal
    vl_desc: CampoDecimal
    ind_mov: IndMov
    cst_icms: CampoInteiro
    cfop: CampoInteiro
    cod_nat: CampoAlphanumerico
    vl_bc_icms: CampoDecimal
    aliq_icms: CampoDecimal
    vl_icms: CampoDecimal
    vl_bc_icms_st: CampoDecimal
    aliq_st: CampoDecimal
    vl_icms_st: CampoDecimal
    ind_apur: IndAPur
    cst_ipi: CampoAlphanumerico
    cod_enq: CampoAlphanumerico
    vl_bc_ipi: CampoDecimal
    aliq_ipi: CampoDecimal
    vl_ipi: CampoDecimal
    cst_pis: CampoInteiro
    vl_bc_pis: CampoDecimal
    aliq_pis_percent: CampoDecimal
    quant_bc_pis: CampoDecimal
    aliq_pis: CampoDecimal
    vl_pis: CampoDecimal
    cst_cofins: CampoInteiro
    vl_bc_cofins: CampoDecimal
    aliq_cofins_percent: CampoDecimal
    quant_bc_cofins: CampoDecimal
    aliq_cofins: CampoDecimal
    vl_cofins: CampoDecimal
    cod_cta: CampoAlphanumerico
    vl_abat_nt: CampoDecimal



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico


class Registro0450(Registro):
    """ TABELA DE INFORMAÇÃO COMPLEMENTAR DO DOCUMENTO FISCAL """

    reg: Literal['0450']
    cod_inf: CampoAlphanumerico
    txt: CampoAlphanumerico



from typing import List, Literal
from pyspedfiscal.campos import CampoAlphanumerico
from pyspedfiscal.models import Registro
from pydantic import Field

from pyspedfiscal.bloco_c.registro_c197 import RegistroC197


class RegistroC195(Registro):
    """ 
    Complemento do Registro Analítico - Observações do Lançamento Fiscal
    (código 01, 1B, 04 e 55) 
    """
    reg: Literal['C195']
    cod_obs: CampoAlphanumerico
    txt_compl: CampoAlphanumerico

    registros_c197: List[RegistroC197] = Field(
        default_factory=list, exclude=True)



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoDecimal


class Registro0221(Registro):
    """ CORRELAÇÃO ENTRE CÓDIGOS DE ITENS COMERCIALIZADOS """

    reg: Literal['0221']
    cod_item_atomico: CampoAlphanumerico
    qtd_contida: CampoDecimal



from typing import Literal
from pyspedfiscal.campos import CampoAlphanumerico, CampoEnumerate, CampoCPF, CampoCNPJ, CampoInteiro
from pyspedfiscal.models import Registro


class IndCarga(CampoEnumerate):
    rodoviario: '0'
    ferroviario: '1'
    rodo_ferroviario: '2'
    aquaviario: '3'
    dutoviario: '4'
    aereo: '5'
    outros: '9'


class RegistroC115(Registro):
    """LOCAL DA COLETA E/OU ENTREGA (CÓDIGO 01, 1B E 04)"""
    reg: Literal['C115']
    ind_carga: IndCarga
    cnpj_col: CampoCNPJ
    ie_col: CampoAlphanumerico
    cpf_col: CampoCPF
    cod_mun_col: CampoInteiro
    cnpj_entg: CampoCNPJ
    ie_entg: CampoAlphanumerico
    cpf_entg: CampoCPF
    cod_mun_entg: CampoInteiro



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoDecimal


class Registro0220(Registro):
    """ FATORES DE CONVERSÃO DE UNIDADES"""

    reg: Literal['0220']
    unid_conv: CampoAlphanumerico
    fat_conv: CampoDecimal
    cod_barra: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico


class Registro0015(Registro):
    """ Dados do Contribuinte Substituto ou Responsável pelo ICMS Destino """

    reg: Literal['0015']
    uf_st: CampoAlphanumerico
    ie_st: CampoAlphanumerico



from typing import List, Literal, Optional
from pyspedfiscal.campos import (
    CampoData, CampoDecimal, CampoInteiro, CampoAlphanumerico, CampoSerie, CampoEnumerate
)
from pydantic import Field
from typing import Literal
from pyspedfiscal.tabelas import CodSit, CodMod
from pyspedfiscal.models import Registro

from pyspedfiscal.bloco_c.registro_c101 import RegistroC101
from pyspedfiscal.bloco_c.registro_c105 import RegistroC105
from pyspedfiscal.bloco_c.registro_c110 import RegistroC110
# from pyspedfiscal.bloco_c.registro_c120 import RegistroC120
# from pyspedfiscal.bloco_c.registro_c130 import RegistroC130
# from pyspedfiscal.bloco_c.registro_c140 import RegistroC140
# from pyspedfiscal.bloco_c.registro_c160 import RegistroC160
from pyspedfiscal.bloco_c.registro_c170 import RegistroC170
# from pyspedfiscal.bloco_c.registro_c180 import RegistroC180
from pyspedfiscal.bloco_c.registro_c190 import RegistroC190
from pyspedfiscal.bloco_c.registro_c195 import RegistroC195


class IndOper(CampoEnumerate):
    """ Indicador do tipo de operação """
    entrada = '0'
    saida = '1'


class IndEmit(CampoEnumerate):
    """ Indicador do emitente do documento fiscal """
    emissao_propria = '0'
    terceiros = '1'


class IndFrt(CampoEnumerate):
    """ Indicador do tipo do frete """
    por_conta_do_remetente = '0'
    por_conta_do_destinataio = '1'
    por_conta_do_terceiro = '2'
    proprio_por_conta_do_remetente = '3'
    proprio_por_conta_do_destinatario = '4'
    sem_ocorrencia = '9'


class IndPgto(CampoEnumerate):
    """ Indicador do tipo de pagamento """
    a_vista = '0'
    a_prazo = '1'
    outros = '2'


class RegistroC100(Registro):
    """  
    NOTA FISCAL (CÓDIGO 01), NOTA FISCAL AVULSA (CÓDIGO 1B), 
    NOTA FISCAL DE PRODUTOR (CÓDIGO 04), NF-e (CÓDIGO 55) e NFC-e (CÓDIGO 65)
    """
    reg: Literal['C100']
    ind_oper: IndOper
    ind_emit: IndEmit
    cod_part: CampoAlphanumerico
    cod_mod: CodMod
    cod_sit: CodSit
    ser: CampoSerie
    num_doc: CampoInteiro
    chv_nfe: CampoInteiro
    dt_doc: CampoData
    dt_e_s: CampoData
    vl_doc: CampoDecimal
    ind_pgto: IndPgto
    vl_desc: CampoDecimal
    vl_abat_nt: CampoDecimal
    vl_merc: CampoDecimal
    ind_frt: IndFrt
    vl_frt: CampoDecimal
    vl_seg: CampoDecimal
    vl_out_da: CampoDecimal
    vl_bc_icms: CampoDecimal
    vl_icms: CampoDecimal
    vl_bc_icms_st: CampoDecimal
    vl_icms_st: CampoDecimal
    vl_ipi: CampoDecimal
    vl_pis: CampoDecimal
    vl_cofins: CampoDecimal
    vl_pis_st: CampoDecimal
    vl_cofins_st: CampoDecimal

    # Registros filhos nivel 3
    registros_c101: Optional[RegistroC101] = Field(exclude=True)
    registros_c105: Optional[RegistroC105] = Field(exclude=True)
    registros_c110: List[RegistroC110] = Field(
        default_factory=list, exclude=True)
    # registros_c120: List[RegistroC120] = Field(default_factory=list)
    # registros_c130: List[RegistroC130] = Field(default_factory=list)
    # registros_c140: List[RegistroC140] = Field(default_factory=list)
    # registros_c160: List[RegistroC160] = Field(default_factory=list)
    registros_c170: List[RegistroC170] = Field(
        default_factory=list, exclude=True)
    # registros_c180: List[RegistroC180] = Field(default_factory=list)
    registros_c190: List[RegistroC190] = Field(
        default_factory=list, exclude=True)
    registros_c195: List[RegistroC195] = Field(
        default_factory=list, exclude=True)



from typing import Literal
from pyspedfiscal.campos import CampoAlphanumerico, CampoInteiro
from pyspedfiscal.models import Registro
from pyspedfiscal.tabelas import CodMod


class RegistroC114(Registro):
    """CUPOM FISCAL REFERENCIADO"""
    reg: Literal['C114']
    cod_mod: CodMod
    ecf_fab: CampoAlphanumerico
    ecf_cx: CampoInteiro
    num_doc: CampoInteiro
    dt_doc: CampoInteiro



from typing import List
from ..models import Bloco
from .registro_c001 import RegistroC001
from .registro_c100 import RegistroC100
from .registro_c101 import RegistroC101
from .registro_c105 import RegistroC105
from .registro_c110 import RegistroC110
from .registro_c111 import RegistroC111
from .registro_c112 import RegistroC112
from .registro_c113 import RegistroC113
from .registro_c114 import RegistroC114
from .registro_c115 import RegistroC115
from .registro_c116 import RegistroC116
from .registro_c120 import RegistroC120
from .registro_c130 import RegistroC130
from .registro_c140 import RegistroC140
from .registro_c160 import RegistroC160
from .registro_c170 import RegistroC170
from .registro_c180 import RegistroC180
from .registro_c190 import RegistroC190
from .registro_c195 import RegistroC195
from .registro_c197 import RegistroC197


class BlocoC(Bloco):
    def __init__(self):
        self.registro_c001: RegistroC001 = None

    @classmethod
    def ler_registros(cls, registros: List[str]) -> 'BlocoC':
        bloco = cls()

        for registro in registros:
            tipo_registro = registro[1:5].upper()

            # Nivel 1 - Registro Pai
            if tipo_registro == 'C001':
                bloco.registro_c001 = RegistroC001.ler_registro(registro)

            # Nivel 2 - Filho de C001
            elif tipo_registro == 'C100':
                registro_c100 = RegistroC100.ler_registro(registro)
                bloco.registro_c001.registros_c100.append(registro_c100)

            # Nivel 3 - Filho de C100
            elif tipo_registro == 'C101':
                registro_c100.registros_c101 = RegistroC101.ler_registro(
                    registro)

            # Nivel 3 - Filho de C100
            elif tipo_registro == 'C105':
                registro_c100.registros_c105 = RegistroC105.ler_registro(
                    registro)

            # Nivel 3 - Filho de C100
            elif tipo_registro == 'C110':
                registro_c110 = RegistroC110.ler_registro(registro)
                registro_c100.registros_c110.append(registro_c110)

            # Nivel 4 - Filho de C110
            elif tipo_registro == 'C111':
                registro_c111 = RegistroC111.ler_registro(registro)
                registro_c110.registros_c111.append(registro_c111)

            # Nivel 4 - Filho de C110
            elif tipo_registro == 'C112':
                registro_c112 = RegistroC112.ler_registro(registro)
                registro_c110.registros_c112.append(registro_c112)

            # Nivel 4 - Filho de C110
            elif tipo_registro == 'C113':
                registro_c113 = RegistroC113.ler_registro(registro)
                registro_c110.registros_c113.append(registro_c113)

            # Nivel 4 - Filho de C110
            elif tipo_registro == 'C114':
                registro_c114 = RegistroC114.ler_registro(registro)
                registro_c110.registros_c114.append(registro_c114)

            # Nivel 4 - Filho de C110
            elif tipo_registro == 'C115':
                registro_c115 = RegistroC115.ler_registro(registro)
                registro_c110.registros_c115.append(registro_c115)

            # Nivel 4 - Filho de C110
            elif tipo_registro == 'C116':
                registro_c116 = RegistroC116.ler_registro(registro)
                registro_c110.registros_c116.append(registro_c116)

            # Nivel 3 - Filho de C100
            elif tipo_registro == 'C170':
                registro_c170 = RegistroC170.ler_registro(registro)
                registro_c100.registros_c170.append(registro_c170)

            # Nivel 3 - Filho de C100
            elif tipo_registro == 'C190':
                registro_c190 = RegistroC190.ler_registro(registro)
                registro_c100.registros_c190.append(registro_c190)

            # Nivel 3 - Filho de C100
            elif tipo_registro == 'C195':
                registro_c195 = RegistroC195.ler_registro(registro)
                registro_c100.registros_c195.append(registro_c195)

            # Nivel 4 - Filho de C195
            elif tipo_registro == 'C197':
                registro_c197 = RegistroC197.ler_registro(registro)
                registro_c195.registros_c197.append(registro_c197)

        return bloco



from typing import Literal
from ..models import Registro
from ..campos import CampoAlphanumerico, CampoEnumerate, CampoInteiro
from typing import List
from .registro_0305 import Registro0305
from pydantic import Field


class IdentMerc(CampoEnumerate):
    """ Identificação do tipo de mercadoria """

    bem = '1'
    componente = '2'


class Registro0300(Registro):
    """ CADASTRO DE BENS OU COMPONENTES DO ATIVO IMOBILIZADO """

    reg: Literal['0300']
    cod_ind_bem: CampoAlphanumerico
    ident_merc: IdentMerc
    descr_item: CampoAlphanumerico
    cod_prnc: CampoAlphanumerico
    cod_cta: CampoAlphanumerico
    nr_parc: CampoInteiro

    registros_0305: List[Registro0305] = Field(
        default_factory=list, exclude=True
    )



from decimal import Decimal
from pyspedfiscal.bloco_c import BlocoC
import unittest


class TestBlocoC(unittest.TestCase):

    def setUp(self) -> None:

        bloco = """
|C001|0|
|C100|1|1|1|55|00|1|1|33220714028298000152550010000000011000147724|27072022|27072022|41961,29|2|0|0|41961,29|0|0|0|0|0|0|0|0|0|0|0|0|0|
|C101|100|20|80|
|C105|0|RJ|
|C110|1|Descrição Complementar Teste|
|C111|Processo Teste|0|
|C112|0|RJ|3|1000|100|01072022|31072022|
|C113|0|0|1|01|001||1|01072022||
|C114|02|1|1|1|01072022|
|C116|59|1|33220714028298000152570010000002110001477205|1|01072022|
|C170|2|1||381,563|UN|2896,06|0|0|060|5101|1653002|0|0|0|0|0|0||||0|0|0|49|2896,06|0,35|0|0|10,14|49|2896,06|0,35|0|0|10,14|4212|0|
|C190|060|5101|0|0|0|0|0|0|0|0||
|C195|1|FECP referente ao diferencial de aliquotas|
|C197|SP10090718|FECP referente ao diferencial de aliquotas||3256,23|0|596,98|0|
|C990|35|""".splitlines()

        self.bloco_c = BlocoC.ler_registros(bloco)

    def test_deve_ler_um_registro_c100(self):

        chv_nfe = self.bloco_c.registro_c001.registros_c100[0].chv_nfe
        vl_doc = self.bloco_c.registro_c001.registros_c100[0].vl_doc
        reg = self.bloco_c.registro_c001.registros_c100[0].reg

        self.assertEqual(reg, 'C100')
        self.assertEqual(
            chv_nfe, 33220714028298000152550010000000011000147724)
        self.assertEqual(
            vl_doc, Decimal('41961.29'))

    def test_deve_ler_um_registro_c101(self):
        vl_fcp_uf_dest = self.bloco_c.registro_c001.registros_c100[0].registros_c101.vl_fcp_uf_dest
        vl_icms_uf_dest = self.bloco_c.registro_c001.registros_c100[
            0].registros_c101.vl_icms_uf_dest
        vl_icms_uf_rem = self.bloco_c.registro_c001.registros_c100[0].registros_c101.vl_icms_uf_rem

        self.assertEqual(vl_fcp_uf_dest, Decimal('100'))
        self.assertEqual(vl_icms_uf_dest, Decimal('20'))
        self.assertEqual(vl_icms_uf_rem, Decimal('80'))
        
    def test_deve_ler_um_registro_c105(self):
        oper = self.bloco_c.registro_c001.registros_c100[0].registros_c105.oper
        uf = self.bloco_c.registro_c001.registros_c100[0].registros_c105.uf

        self.assertEqual(oper, '0')
        self.assertEqual(uf, 'RJ')


if __name__ == '__main__':
    unittest.main()



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoInteiro


class Registro9999(Registro):
    """ Abertura do Bloco 9"""
    reg: Literal['9999']
    qtd_lin: CampoInteiro



from pyspedfiscal import SpedFiscal
import unittest
from pathlib import Path


arquivo_teste = Path(__file__).parent / 'arquivos' / 'arquivo_teste.txt'


class TestSpedFiscal(unittest.TestCase):

    def test_deve_ler_um_arquivo_sped_fiscal(self):
        sped_fiscal = SpedFiscal.importar(arquivo_teste)
        assert sped_fiscal.bloco_0.registro_0000.cnpj == 14028298000152

    def test_deve_validar_um_arquivo_fiscal(self):

        esperado = True

        sped_fiscal = SpedFiscal()
        eh_valido = sped_fiscal.validar(arquivo_teste)

        self.assertEqual(eh_valido, esperado)


if __name__ == '__main__':
    unittest.main()



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico


class Registro0190(Registro):
    """ IDENTIFICAÇÃO DAS UNIDADES DE MEDIDA """

    reg: Literal['0190']
    unid: CampoAlphanumerico
    descr: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoInteiro
from typing import List
from .registro_0175 import Registro0175
from pydantic import Field


class Registro0150(Registro):
    """ Tabela de Cadastro do Participante """

    reg: Literal['0150']
    cod_part: CampoAlphanumerico
    nome: CampoAlphanumerico
    cod_pais: CampoInteiro
    cnpj: CampoInteiro
    cpf: CampoInteiro
    ie: CampoAlphanumerico
    cod_mun: CampoInteiro
    suframa: CampoAlphanumerico
    end: CampoAlphanumerico
    num: CampoAlphanumerico
    compl: CampoAlphanumerico
    bairro: CampoAlphanumerico

    # Registros filhos nivel 3
    registros_0175: List[Registro0175] = Field(
        default_factory=list, exclude=True)



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoDecimal


class RegistroE115(Registro):
    """ 
    INFORMAÇÕES ADICIONAIS DA APURAÇÃO – VALORES
    DECLARATÓRIOS 
    """

    reg: Literal['E115']
    cod_inf_adic: CampoAlphanumerico
    vl_inf_adic: CampoDecimal
    descr_compl_aj: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoInteiro, CampoEnumerate, CampoData


class CodigoNatCC(CampoEnumerate):
    """ Código da natureza da conta/grupo de contas """
    passivo = '01'
    ativo = '02'
    patrimonio_liquido = '03'
    contas_resultado = '04'
    contas_compensacao = '05'
    outras = '09'


class IndCTA(CampoEnumerate):
    """ Indicador do tipo de conta """
    analitica = 'A'
    sintetica = 'S'


class Registro0500(Registro):
    """ PLANO DE CONTAS CONTÁBEIS """

    reg: Literal['0500']
    dt_alt: CampoData
    cod_nat_cc: CodigoNatCC
    ind_cta: IndCTA
    nivel: CampoInteiro
    cod_cta: CampoAlphanumerico
    nome_cta: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoInteiro


class Registro0005(Registro):
    """ Dados Complementares da entidade """

    reg: Literal['0005']
    fantasia: CampoAlphanumerico
    cep: CampoInteiro
    end: CampoAlphanumerico
    num: CampoAlphanumerico
    compl: CampoAlphanumerico
    bairro: CampoAlphanumerico
    fone: CampoAlphanumerico
    fax: CampoAlphanumerico
    email: CampoAlphanumerico



from typing import List
from ..models import Bloco
from .registro_e001 import RegistroE001
from .registro_e100 import RegistroE100
from .registro_e110 import RegistroE110
from .registro_e115 import RegistroE115


class BlocoE(Bloco):
    def __init__(self):
        self.registro_e001: RegistroE001 = None

    @classmethod
    def ler_registros(cls, registros: List[str]) -> 'BlocoE':
        bloco = cls()

        for registro in registros:
            tipo_registro = registro[1:5].upper()

            # Nivel 1 - Registro Pai
            if tipo_registro == 'E001':
                bloco.registro_e001 = RegistroE001.ler_registro(registro)

            # Nivel 2 - Filho de E001
            if tipo_registro == 'E100':
                bloco.registro_e001.registros_e100.append(
                    RegistroE100.ler_registro(registro)
                )

            # Nivel 3 - Filho de E100
            if tipo_registro == 'E110':
                bloco.registro_e001.registros_e100[-1].registros_e110.append(
                    RegistroE110.ler_registro(registro)
                )

            # Nivel 4 - Filho de E110
            if tipo_registro == 'E115':
                bloco.registro_e001.registros_e100[-1].registros_e110[-1].registros_c115.append(
                    RegistroE115.ler_registro(registro)
                )

        return bloco



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoInteiro
from typing import List
from pydantic import Field


class Registro0100(Registro):
    """ Dados do Contabilista """

    reg: Literal['0100']
    nome: CampoAlphanumerico
    cpf: CampoInteiro
    crc: CampoAlphanumerico
    cnpj: CampoInteiro
    cep: CampoInteiro
    end: CampoAlphanumerico
    num: CampoAlphanumerico
    compl: CampoAlphanumerico
    bairro: CampoAlphanumerico
    fone: CampoAlphanumerico
    fax: CampoAlphanumerico
    email: CampoAlphanumerico
    cod_mun: CampoInteiro



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoData


class Registro0600(Registro):
    """ CENTRO DE CUSTOS """

    reg: Literal['0600']
    dt_alt: CampoData
    cod_ccus: CampoAlphanumerico
    ccus: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico


class Registro0460(Registro):
    """ TABELA DE OBSERVAÇÕES DO LANÇAMENTO FISCAL """

    reg: Literal['0460']
    cod_obs: CampoAlphanumerico
    txt: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoDecimal


class Registro0210(Registro):
    """ CONSUMO ESPECÍFICO PADRONIZADO (VÁLIDO ATÉ 31/12/2021) """

    reg: Literal['0210']
    cod_item_comp: CampoAlphanumerico
    qtd_comp: CampoDecimal
    perda: CampoDecimal



from typing import Literal
from typing import List
from pydantic import Field
from ..models import Registro
from ..campos import CampoEnumerate, CampoAlphanumerico, CampoDecimal, CampoInteiro
from .registro_0205 import Registro0205
from .registro_0206 import Registro0206
from .registro_0210 import Registro0210
from .registro_0220 import Registro0220
from .registro_0221 import Registro0221


class TipoItem(CampoEnumerate):
    revenda = '00'
    materia_prima = '01'
    embalagem = '02'
    produto_em_processo = '03'
    produto_acabado = '04'
    subproduto = '05'
    produto_intermediario = '06'
    uso_e_consumo = '07'
    imobilizado = '08'
    servico = '09'
    outros_insumos = '10'
    outras = '99'


class Registro0200(Registro):
    """ TABELA DE IDENTIFICAÇÃO DO ITEM (PRODUTO E SERVIÇOS) """

    reg: Literal['0200']
    cod_item: CampoAlphanumerico
    descr_item: CampoAlphanumerico
    cod_barra: CampoAlphanumerico
    cod_ant_item: CampoAlphanumerico
    unid_inv: CampoAlphanumerico
    tipo_item: TipoItem
    cod_ncm: CampoAlphanumerico
    ex_ipi: CampoAlphanumerico
    cod_gen: CampoAlphanumerico  # TODO: Tabela 4.2.1.
    # TODO: Anexo I da Lei Complementar Federal nº 116/03.
    cod_lst: CampoAlphanumerico
    aliq_icms: CampoDecimal
    cest: CampoInteiro

    # Registros filhos nivel 3
    registros_0205: List[Registro0205] = Field(
        default_factory=list, exclude=True)
    registros_0206: List[Registro0206] = Field(
        default_factory=list, exclude=True)
    registros_0210: List[Registro0210] = Field(
        default_factory=list, exclude=True)
    registros_0220: List[Registro0220] = Field(
        default_factory=list, exclude=True)
    registros_0221: List[Registro0221] = Field(
        default_factory=list, exclude=True)



from typing import Literal
from pyspedfiscal.models import Registro
from ..tabelas import ClassifContribIpi


class Registro0002(Registro):
    """ Classificação do Estabelecimento Industrial ou Equiparado a Industrial """

    reg: Literal['0002']
    clas_estab_ind: ClassifContribIpi



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoEnumerate


class IndMov(CampoEnumerate):
    """ Indicador de movimento """
    com_movimento = '0'
    sem_movimento = '1'


class Registro9001(Registro):
    """ Abertura do Bloco 9"""
    reg: Literal['9001']
    ind_mov: IndMov



from typing import Literal
from pyspedfiscal.campos import CampoAlphanumerico, CampoEnumerate, CampoDecimal, CampoData
from pyspedfiscal.models import Registro


class CodDA(CampoEnumerate):
    """Código do modelo do documento de arrecadação"""
    documento_estadual = '0'
    gnre = '1'


class RegistroC112(Registro):
    """DOCUMENTO DE ARRECADAÇÃO REFERENCIADO"""
    reg: Literal['C112']
    cod_da: CodDA
    uf: CampoAlphanumerico
    num_da: CampoAlphanumerico
    cod_aut: CampoAlphanumerico
    vl_da: CampoDecimal
    dt_vcto: CampoData
    dt_pgto: CampoData



from __future__ import annotations
from abc import ABC, abstractmethod
from datetime import datetime, date
from decimal import Decimal, InvalidOperation
from typing import Literal, Union, Optional
from pydantic.fields import ModelField
from pyspedfiscal.exception import ValorCampoInvalido
from enum import Enum


class Campo(ABC):
    @classmethod
    @abstractmethod
    def __get_validators__(cls):
        pass

    @classmethod
    @abstractmethod
    def validate(cls, v, field: ModelField) -> Union[date, Literal['']]:
        pass


class CampoData(date, Campo):
    """ Representa uma data """

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, field: ModelField) -> Optional[date]:

        campo = field.name

        if isinstance(v, str):
            if v == '':
                return None

            try:
                return datetime.strptime(v, '%d%m%Y').date()
            except ValueError:
                raise ValorCampoInvalido(campo, v)

        elif isinstance(v, date):
            return v
        elif isinstance(v, datetime):
            return v.date()
        else:
            raise ValorCampoInvalido(campo, v)


class CampoDecimal(Decimal, Campo):
    """ Representa um numero decimal """
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, field: ModelField) -> Optional[Decimal]:

        campo = field.name

        if isinstance(v, (float, Decimal, int)):
            return Decimal(v)
        elif isinstance(v, str):
            if v == '':
                return None

            try:
                return Decimal(v.replace(',', '.'))
            except InvalidOperation:
                raise ValorCampoInvalido(campo, v)

        else:
            raise ValorCampoInvalido(campo, v)


class CampoInteiro(int, Campo):
    """ Representa um numero decimal """
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, field: ModelField) -> Optional[int]:

        campo = field.name

        if isinstance(v, int):
            return v

        elif isinstance(v, str):
            if v == '':
                return None

            try:
                return int(v)
            except ValueError:
                raise ValorCampoInvalido(campo, v)

        else:
            raise ValorCampoInvalido(campo, v)


class CampoAlphanumerico(str, Campo):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, field: ModelField) -> Optional[None]:
        campo = field.name

        if isinstance(v, str):
            if v == '':
                return None

            return v
        else:
            raise ValorCampoInvalido(campo, v)


class CampoEnumerate(str, Enum):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, field: ModelField) -> Optional[str]:

        campo = field.name

        if v == '':
            return None

        if v not in list(cls):
            raise ValorCampoInvalido(campo, v)

        return cls(v)


class CampoSerie(CampoAlphanumerico):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate
        yield cls._converter

    @classmethod
    def _converter(cls, valor: str, field: ModelField):
        _valor = valor.strip()

        if _valor.isalnum():
            return _valor.zfill(3)

        return _valor


class CampoCNPJ(CampoInteiro):
    pass


class CampoCPF(CampoInteiro):
    pass


class CampoCNPJouCPF(CampoInteiro):
    pass


class CampoChaveAcesso(CampoAlphanumerico):
    pass


class OutroCampo(Campo):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v: str):
        pass



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico


class Registro0400(Registro):
    """ TABELA DE NATUREZA DA OPERAÇÃO/PRESTAÇÃO """

    reg: Literal['0400']
    cod_nat: CampoAlphanumerico
    descr_nat: CampoAlphanumerico



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoData


class Registro0205(Registro):
    """ ALTERAÇÃO DO ITEM """

    reg: Literal['0205']
    descr_ant_item: CampoAlphanumerico
    dt_ini: CampoData
    dt_fim: CampoData
    cod_ant_item: CampoAlphanumerico



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    readme = fh.read()

setup(
    name='pyspedfiscal',
    version='0.2.7',
    url='https://github.com/rodolfoscarp-jr/pyspedfiscal',
    license='MIT License',
    author='Rodolfo Scarp',
    long_description=readme,
    long_description_content_type="text/markdown",
    author_email='rodolfoscarp@gmail.com',
    keywords='spedfiscal',
    description=u'Serializa um arquivo do tipo SPED Fiscal',
    packages=find_packages(exclude=['test*']),
    install_requires=['pydantic']
)



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoEnumerate
from typing import List
from pydantic import Field

from .registro_0002 import Registro0002
from .registro_0005 import Registro0005
from .registro_0015 import Registro0015
from .registro_0100 import Registro0100
from .registro_0150 import Registro0150
from .registro_0190 import Registro0190
from .registro_0200 import Registro0200
from .registro_0300 import Registro0300
from .registro_0400 import Registro0400
from .registro_0450 import Registro0450
from .registro_0460 import Registro0460
from .registro_0500 import Registro0500
from .registro_0600 import Registro0600


class IndMov(CampoEnumerate):
    """ Indicador de movimento """
    com_movimento = '0'
    sem_movimento = '1'


class Registro0001(Registro):
    """ ABERTURA DO BLOCO 0 """

    reg: Literal['0001']
    ind_mov: IndMov

    # Registros filhos nivel 2
    registro_0002: Registro0002 = Field(
        default=None, exclude=True)
    registro_0005: Registro0005 = Field(
        default=None, exclude=True)
    registros_0015: List[Registro0015] = Field(
        default_factory=list, exclude=True)
    registro_0100: Registro0100 = Field(
        default=None, exclude=True)
    registros_0150: List[Registro0150] = Field(
        default_factory=list, exclude=True)
    registros_0190: List[Registro0190] = Field(
        default_factory=list, exclude=True)
    registros_0200: List[Registro0200] = Field(
        default_factory=list, exclude=True)
    registros_0300: List[Registro0300] = Field(
        default_factory=list, exclude=True)
    registros_0400: List[Registro0400] = Field(
        default_factory=list, exclude=True)
    registros_0450: List[Registro0450] = Field(
        default_factory=list, exclude=True)
    registros_0460: List[Registro0460] = Field(
        default_factory=list, exclude=True)
    registros_0500: List[Registro0500] = Field(
        default_factory=list, exclude=True)
    registros_0600: List[Registro0600] = Field(
        default_factory=list, exclude=True)



from datetime import date
from decimal import Decimal
from pyspedfiscal.bloco_0 import Bloco0
from pyspedfiscal.bloco_0.registro_0000 import IndAtiv
from pyspedfiscal.bloco_0.registro_0500 import IndCTA, CodigoNatCC
import unittest
from pyspedfiscal import tabelas


class TestBloco0(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:

        bloco = """
|0000|016|0|01072022|31072022|EMPRESA TESTE|14028298000152||RJ|42497045|3300233|||A|0|
|0001|0|
|0002|00|
|0005|EMPRESA TESTE|25821270|RUA EMPRESA TESTE|1111||BAIRRO EMPRESA TESTE|2499999999|||
|0015|RJ|14201122|
|0100|CONTADOR TESTE|25142702000|1-SP-111111/O-1||25805290|RUA CONTADOR|111|ENDERECO CONTADOR|BAIRRO CONTADOR|2499999999||emailcontador@contador.com.br|3550308|
|0150|1|FORNECEDOR 1|1058|03446003000141|||3301702||ENDERECO FORNECEDOR 1|1111||BAIRRO FORNECEDOR 1|
|0175|10072022|11|155|
|0150|2|FORNECEDOR 2|1058|97509345000107|||3550308||ENDERECO FORNECEDOR 1|2222||BAIRRO FORNECEDOR 2|
|0175|10072022|11|154|
|0150|3|FORNECEDOR 3|1058|05929208000103|||3550308||ENDERECO FORNECEDOR 1|3333||BAIRRO FORNECEDOR 3|
|0175|10072022|11|152|
|0150|4|FORNECEDOR 4|1058|13992567000133|||3550308||ENDERECO FORNECEDOR 1|4444||BAIRRO FORNECEDOR 4|
|0175|10072022|11|153|
|0150|5|FORNECEDOR 5|1058|38284654000167|||3550308||ENDERECO FORNECEDOR 1|5555||BAIRRO FORNECEDOR 5|
|0175|10072022|11|145|
|0175|11072022|03|FORNECEDOR 11|
|0190|KG|KILO|
|0190|UN|UNIDADE|
|0200|1|PRODUTO 1|||UN|00|27101921||||||
|0205|PRODUTO 1 ANTERIOR|01012022|31012022|11|
|0206|110101001|
|0210|1|10|1|
|0220|KG|15|PRODUTO|
|0200|2|PRODUTO 2|||KG|00|27101921||||||
|0200|3|PRODUTO 3|||KG|00|27101921||||||
|0200|4|PRODUTO 4|||KG|00|27101921||||||
|0200|5|PRODUTO 5|||KG|00|27101921||||||
|0300|1|1|BEM TESTE|1|1|5|
|0305|TESTE|TESTE|5|
|0400|1653002|Natureza Teste|
|0450|1|Informacao Complementar Teste|
|0460|1|Observacao Teste|
|0500|01012022|01|A|1|1|CONTA TESTE|
|0600|01012022|TESTE|TESTE|
|0990|32|""".splitlines()

        cls.bloco_0 = Bloco0.ler_registros(bloco)

    def test_deve_ler_um_registro_0000(self):
        registro_0000 = self.bloco_0.registro_0000

        # |0000|016|0|01072022|31072022|EMPRESA TESTE|14028298000152||RJ|42497045|3300233|||A|0|

        self.assertEqual(registro_0000.reg, '0000')
        self.assertEqual(registro_0000.cod_ver, '016')
        self.assertEqual(registro_0000.cod_fin, '0')
        self.assertEqual(registro_0000.dt_ini, date(2022, 7, 1))
        self.assertEqual(registro_0000.dt_fin, date(2022, 7, 31))
        self.assertEqual(registro_0000.nome, 'EMPRESA TESTE')
        self.assertEqual(registro_0000.cnpj, 14028298000152)
        self.assertEqual(registro_0000.cpf, None)
        self.assertEqual(registro_0000.uf, 'RJ')
        self.assertEqual(registro_0000.ie, '42497045')
        self.assertEqual(registro_0000.cod_mun, 3300233)
        self.assertEqual(registro_0000.im, None)
        self.assertEqual(registro_0000.suframa, None)
        self.assertEqual(registro_0000.ind_perfil, 'A')
        self.assertEqual(registro_0000.ind_ativ, IndAtiv.industrial)

    def test_deve_ler_um_registro_0001(self):
        registro_0001 = self.bloco_0.registro_0001

        reg = registro_0001.reg
        ind_mov = registro_0001.ind_mov

        # |0001|0|

        self.assertEqual(reg, '0001')
        self.assertEqual(ind_mov, '0')

    def test_deve_ler_um_registro_0002(self):
        registro_0002 = self.bloco_0.registro_0001.registro_0002

        reg = registro_0002.reg
        clas_estab_ind = registro_0002.clas_estab_ind

        # |0002|00|

        self.assertEqual(reg, '0002')
        self.assertEqual(clas_estab_ind, tabelas.ClassifContribIpi.industrial_transformacao)

    def test_deve_ler_um_registro_0005(self):
        registro_0005 = self.bloco_0.registro_0001.registro_0005

        reg = registro_0005.reg
        fantasia = registro_0005.fantasia
        cep = registro_0005.cep
        end = registro_0005.end
        num = registro_0005.num
        compl = registro_0005.compl
        bairro = registro_0005.bairro
        fone = registro_0005.fone
        fax = registro_0005.fax
        email = registro_0005.email

        # |0005|EMPRESA TESTE|25821270|RUA EMPRESA TESTE|1111||BAIRRO EMPRESA TESTE|2499999999|||

        self.assertEqual(reg, '0005')
        self.assertEqual(fantasia, 'EMPRESA TESTE')
        self.assertEqual(cep, 25821270)
        self.assertEqual(end, 'RUA EMPRESA TESTE')
        self.assertEqual(num, '1111')
        self.assertEqual(compl, None)
        self.assertEqual(bairro, 'BAIRRO EMPRESA TESTE')
        self.assertEqual(fone, '2499999999')
        self.assertEqual(fax, None)
        self.assertEqual(email, None)

    def test_deve_ler_um_registro_0015(self):
        registros_0015 = self.bloco_0.registro_0001.registros_0015[0]

        # |0015|RJ|14201122|

        self.assertEqual(registros_0015.reg, '0015')
        self.assertEqual(registros_0015.uf_st, 'RJ')
        self.assertEqual(registros_0015.ie_st, '14201122')

    def test_deve_ler_um_registro_0100(self):
        registro_0100 = self.bloco_0.registro_0001.registro_0100

        # |0100|CONTADOR TESTE|25142702000|1-SP-111111/O-1||25805290|RUA CONTADOR|111|ENDERECO CONTADOR|BAIRRO CONTADOR|2499999999||emailcontador@contador.com.br|3550308|

        self.assertEqual(registro_0100.reg, '0100')
        self.assertEqual(registro_0100.nome, 'CONTADOR TESTE')
        self.assertEqual(registro_0100.cpf, 25142702000)
        self.assertEqual(registro_0100.crc, '1-SP-111111/O-1')
        self.assertEqual(registro_0100.cnpj, None)
        self.assertEqual(registro_0100.cep, 25805290)
        self.assertEqual(registro_0100.end, 'RUA CONTADOR')
        self.assertEqual(registro_0100.num, '111')
        self.assertEqual(registro_0100.compl, 'ENDERECO CONTADOR')
        self.assertEqual(registro_0100.bairro, 'BAIRRO CONTADOR')
        self.assertEqual(registro_0100.fone, '2499999999')
        self.assertEqual(registro_0100.fax, None)
        self.assertEqual(registro_0100.email, 'emailcontador@contador.com.br')
        self.assertEqual(registro_0100.cod_mun, 3550308)

    def test_deve_ler_um_registro_0150(self):
        registro_0150 = self.bloco_0.registro_0001.registros_0150[0]

        # |0150|1|FORNECEDOR 1|1058|03446003000141|||3301702||ENDERECO FORNECEDOR 1|1111||BAIRRO FORNECEDOR 1|

        self.assertEqual(registro_0150.reg, '0150')
        self.assertEqual(registro_0150.cod_part, '1')
        self.assertEqual(registro_0150.nome, 'FORNECEDOR 1')
        self.assertEqual(registro_0150.cod_pais, 1058)
        self.assertEqual(registro_0150.cnpj, 3446003000141)
        self.assertEqual(registro_0150.cpf, None)
        self.assertEqual(registro_0150.ie, None)
        self.assertEqual(registro_0150.cod_mun, 3301702)
        self.assertEqual(registro_0150.suframa, None)
        self.assertEqual(registro_0150.end, 'ENDERECO FORNECEDOR 1')
        self.assertEqual(registro_0150.num, '1111')
        self.assertEqual(registro_0150.compl, None)
        self.assertEqual(registro_0150.bairro, 'BAIRRO FORNECEDOR 1')

    def test_deve_ler_um_registro_0175(self):
        registro_0175_1 = self.bloco_0.registro_0001.registros_0150[-1].registros_0175[0]
        registro_0175_2 = self.bloco_0.registro_0001.registros_0150[-1].registros_0175[1]

        # |0175|10072022|11|145|
        # |0175|11072022|03|FORNECEDOR 11|

        self.assertEqual(registro_0175_1.reg, '0175')
        self.assertEqual(registro_0175_1.dt_alt, date(2022, 7, 10))
        self.assertEqual(registro_0175_1.nr_campo, '11')
        self.assertEqual(registro_0175_1.cont_ant, '145')

        self.assertEqual(registro_0175_2.reg, '0175')
        self.assertEqual(registro_0175_2.dt_alt, date(2022, 7, 11))
        self.assertEqual(registro_0175_2.nr_campo, '03')
        self.assertEqual(registro_0175_2.cont_ant, 'FORNECEDOR 11')

    def test_deve_ler_um_registro_0190(self):
        registro_0190 = self.bloco_0.registro_0001.registros_0190[0]

        # |0190|KG|KILO|

        self.assertEqual(registro_0190.reg, '0190')
        self.assertEqual(registro_0190.unid, 'KG')
        self.assertEqual(registro_0190.descr, 'KILO')

    def test_deve_ler_um_registro_0200(self):
        registro_0200 = self.bloco_0.registro_0001.registros_0200[1]

        # |0200|2|PRODUTO 2|||KG|00|27101921||||||

        self.assertEqual(registro_0200.reg, '0200')
        self.assertEqual(registro_0200.cod_item, '2')
        self.assertEqual(registro_0200.descr_item, 'PRODUTO 2')
        self.assertEqual(registro_0200.cod_barra, None)
        self.assertEqual(registro_0200.cod_ant_item, None)
        self.assertEqual(registro_0200.unid_inv, 'KG')
        self.assertEqual(registro_0200.tipo_item, '00')
        self.assertEqual(registro_0200.cod_ncm, '27101921')
        self.assertEqual(registro_0200.ex_ipi, None)
        self.assertEqual(registro_0200.cod_gen, None)
        self.assertEqual(registro_0200.cod_lst, None)
        self.assertEqual(registro_0200.aliq_icms, None)
        self.assertEqual(registro_0200.cest, None)

    def test_deve_ler_um_registro_0205(self):
        registros_0205 = self.bloco_0.registro_0001.registros_0200[0].registros_0205[0]

        # |0205|PRODUTO 1 ANTERIOR|01012022|31012022|11|

        self.assertEqual(registros_0205.reg, '0205')
        self.assertEqual(registros_0205.descr_ant_item, 'PRODUTO 1 ANTERIOR')
        self.assertEqual(registros_0205.dt_ini, date(2022, 1, 1))
        self.assertEqual(registros_0205.dt_fim, date(2022, 1, 31))
        self.assertEqual(registros_0205.cod_ant_item, '11')

    def test_deve_ler_um_registro_0206(self):
        registros_0206 = self.bloco_0.registro_0001.registros_0200[0].registros_0206[0]

        # |0206|110101001|

        self.assertEqual(registros_0206.reg, '0206')
        self.assertEqual(registros_0206.cod_comb, '110101001')

    def test_deve_ler_um_registro_0210(self):
        registros_0210 = self.bloco_0.registro_0001.registros_0200[0].registros_0210[0]

        # |0210|1|10|1|

        self.assertEqual(registros_0210.reg, '0210')
        self.assertEqual(registros_0210.cod_item_comp, '1')
        self.assertEqual(registros_0210.qtd_comp, Decimal('10'))
        self.assertEqual(registros_0210.perda, Decimal('1'))

    def test_deve_ler_um_registro_0220(self):
        registros_0220 = self.bloco_0.registro_0001.registros_0200[0].registros_0220[0]

        # |0220|KG|15|PRODUTO|

        self.assertEqual(registros_0220.reg, '0220')
        self.assertEqual(registros_0220.unid_conv, 'KG')
        self.assertEqual(registros_0220.fat_conv, Decimal('15'))
        self.assertEqual(registros_0220.cod_barra, 'PRODUTO')

    def test_deve_ler_um_registro_0221(self):

        # PVA não implementou este registro
        registros_0221 = self.bloco_0.registro_0001.registros_0200[0].registros_0221

        self.assertEqual(len(registros_0221), 0)

    def test_deve_ler_um_registro_0300(self):
        registros_0300 = self.bloco_0.registro_0001.registros_0300[0]

        #   |0300|1|1|BEM TESTE|1|1|5|

        self.assertEqual(registros_0300.reg, '0300')
        self.assertEqual(registros_0300.cod_ind_bem, '1')
        self.assertEqual(registros_0300.ident_merc, '1')
        self.assertEqual(registros_0300.descr_item, 'BEM TESTE')
        self.assertEqual(registros_0300.cod_prnc, '1')
        self.assertEqual(registros_0300.cod_cta, '1')
        self.assertEqual(registros_0300.nr_parc, 5)

    def test_deve_ler_um_registro_0305(self):
        registros_0305 = self.bloco_0.registro_0001.registros_0300[0].registros_0305[0]

        #   |0305|TESTE|TESTE|5|

        self.assertEqual(registros_0305.reg, '0305')
        self.assertEqual(registros_0305.cod_ccus, 'TESTE')
        self.assertEqual(registros_0305.func, 'TESTE')
        self.assertEqual(registros_0305.vida_util, 5)

    def test_deve_ler_um_registro_0400(self):
        registros_0400 = self.bloco_0.registro_0001.registros_0400[0]

        # |0400|1653002|Natureza Teste|

        self.assertEqual(registros_0400.reg, '0400')
        self.assertEqual(registros_0400.cod_nat, '1653002')
        self.assertEqual(registros_0400.descr_nat, 'Natureza Teste')

    def test_deve_ler_um_registro_0450(self):
        registros_0450 = self.bloco_0.registro_0001.registros_0450[0]

        # |0450|1|Informacao Complementar Teste|

        self.assertEqual(registros_0450.reg, '0450')
        self.assertEqual(registros_0450.cod_inf, '1')
        self.assertEqual(registros_0450.txt, 'Informacao Complementar Teste')

    def test_deve_ler_um_registro_0460(self):
        registro_0460 = self.bloco_0.registro_0001.registros_0460[0]

        # |0460|1|Observacao Teste|

        self.assertEqual(registro_0460.reg, '0460')
        self.assertEqual(registro_0460.cod_obs, '1')
        self.assertEqual(registro_0460.txt, 'Observacao Teste')

    def test_deve_ler_um_registro_0500(self):
        registros_0500 = self.bloco_0.registro_0001.registros_0500[0]

        # |0500|01012022|01|A|1|1|CONTA TESTE|

        self.assertEqual(registros_0500.reg, '0500')
        self.assertEqual(registros_0500.dt_alt, date(2022, 1, 1))
        self.assertEqual(registros_0500.cod_nat_cc, CodigoNatCC.passivo)
        self.assertEqual(registros_0500.ind_cta, IndCTA.analitica)
        self.assertEqual(registros_0500.nivel, 1)
        self.assertEqual(registros_0500.cod_cta, '1')
        self.assertEqual(registros_0500.nome_cta, 'CONTA TESTE')

    def test_deve_ler_um_registro_0600(self):
        registros_0600 = self.bloco_0.registro_0001.registros_0600[0]

        # |0600|01012022|TESTE|TESTE|

        self.assertEqual(registros_0600.reg, '0600')
        self.assertEqual(registros_0600.dt_alt, date(2022, 1, 1))
        self.assertEqual(registros_0600.cod_ccus, 'TESTE')
        self.assertEqual(registros_0600.ccus, 'TESTE')

    def test_deve_ler_um_registro_0990(self):
        registros_0990 = self.bloco_0.registro_0990

        # |0990|32|

        self.assertEqual(registros_0990.reg, '0990')
        self.assertEqual(registros_0990.qtd_lin_0, 32)


if __name__ == '__main__':
    unittest.main()



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoAlphanumerico, CampoInteiro


class Registro0305(Registro):
    """ INFORMAÇÃO SOBRE A UTILIZAÇÃO DO BEM """

    reg: Literal['0305']
    cod_ccus: CampoAlphanumerico
    func: CampoAlphanumerico
    vida_util: CampoInteiro



from typing import Literal
from pyspedfiscal.models import Registro
from pyspedfiscal.campos import CampoInteiro


class Registro0990(Registro):
    """  """

    reg: Literal['0990']
    qtd_lin_0: CampoInteiro



from decimal import Decimal
from pyspedfiscal.bloco_e import BlocoE
import unittest
from datetime import date


class TestBlocoE(unittest.TestCase):

    def setUp(self) -> None:

        bloco = """
|E001|0|
|E100|01072022|31072022|
|E110|0|0|0|0|16,63|2984,9|0|0|0|0|0|0|3001,53|0|
|E115|RJ000001|100|Teste1|
|E300|RJ|01072022|31072022|
|E310|0|0|20|0|0|0|20|0|20|0|0|0|100|0|0|0|100|0|100|0|0|
|E300|SP|01072022|31072022|
|E310|0|0|80|0|0|0|80|0|80|0|0|0|0|0|0|0|0|0|0|0|0|
|E500|0|01072022|31072022|
|E520|0|0|0|0|0|0|0|
|E990|11|""".splitlines()

        self.bloco_e = BlocoE.ler_registros(bloco)

    def test_deve_ler_um_registro_e001(self):
        registro_e001 = self.bloco_e.registro_e001

        reg = registro_e001.reg
        ind_mov = registro_e001.ind_mov

        self.assertEqual(reg, 'E001')
        self.assertEqual(ind_mov, '0')

    def test_deve_ler_um_registro_e100(self):
        resgitro_e100 = self.bloco_e.registro_e001.registros_e100[0]

        reg = resgitro_e100.reg
        dt_ini = resgitro_e100.dt_ini
        dt_fin = resgitro_e100.dt_fin

        self.assertEqual(reg, 'E100')
        self.assertEqual(dt_ini, date(2022, 7, 1))
        self.assertEqual(dt_fin, date(2022, 7, 31))

    def test_deve_ler_um_registro_e110(self):
        registro_e110 = self.bloco_e.registro_e001.registros_e100[0].registros_e110[0]

        reg = registro_e110.reg
        vl_tot_creditos = registro_e110.vl_tot_creditos
        vl_aj_creditos = registro_e110.vl_aj_creditos
        vl_sld_credor_transportar = registro_e110.vl_sld_credor_transportar

        self.assertEqual(reg, 'E110')
        self.assertEqual(vl_tot_creditos, Decimal('16.63'))
        self.assertEqual(vl_aj_creditos, Decimal('2984.9'))
        self.assertEqual(vl_sld_credor_transportar, Decimal('3001.53'))

    def test_deve_ler_um_registro_e115(self):
        registro_c115 = self.bloco_e.registro_e001.registros_e100[
            0].registros_e110[0].registros_c115[0]

        reg = registro_c115.reg
        cod_inf_adic = registro_c115.cod_inf_adic
        vl_inf_adic = registro_c115.vl_inf_adic
        descr_compl_aj = registro_c115.descr_compl_aj

        self.assertEqual(reg, 'E115')
        self.assertEqual(cod_inf_adic, 'RJ000001')
        self.assertEqual(vl_inf_adic, Decimal('100'))
        self.assertEqual(descr_compl_aj, 'Teste1')


if __name__ == '__main__':
    unittest.main()
